/*==========================================================================*\
|                                                                            |
| Replicator.c                                                               |
|                                                                            |
| JTAG Replicator for the MSP430 Flash-based family devices                  |
|                                                                            |
| Key features:                                                              |
|  Supports JTAG communication to all MSP430 Flash Devices                  |
|  Max. code size of target program: 57kB                                   |
|  Programming speed: 60kB/10sec (6kB/sec)                                  |
|  Fast Verification and Erase Check: 60kB/350ms                            |
|  Supports Programming of JTAG Access Protection Fuse                      | 
|                                                                            |
|----------------------------------------------------------------------------|
| Project:              MSP430 Replicator                                    |
| Developed using:      IAR Embedded Workbench 2.31C                         |
|----------------------------------------------------------------------------|
| Author:               FRGR                                                 |
| Version:              1.2                                                  |
| Initial Version:      04-17-02                                             |
| Last Change:          06-04-02                                             |
|----------------------------------------------------------------------------|
| Version history:                                                           |
| 1.0 04/02 FRGR        Initial version.                                     |
| 1.1 04/02 FRGR        Included SPI mode to speed up shifting function by 2.|
| 1.2 06/02 ALB2        Formatting changes, added comments.                  |
|----------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments Germany                                 |
\*==========================================================================*/

// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted 
// by Texas Instruments is distributed as "freeware".  You may, 
// only under TI's copyright in the Program, use and modify the 
// Program without any charge or restriction.  You may 
// distribute to third parties, provided that you transfer a 
// copy of this license to the third party and the third party 
// agrees to these terms by its first use of the Program. You 
// must reproduce the copyright notice and any other legend of 
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains 
// copyrighted material, trade secrets and other TI proprietary 
// information and is protected by copyright laws, 
// international copyright treaties, and trade secret laws, as 
// well as other intellectual property laws.  To protect TI's 
// rights in the Program, you agree not to decompile, reverse 
// engineer, disassemble or otherwise translate any object code 
// versions of the Program to a human-readable form.  You agree 
// that in no event will you alter, remove or destroy any 
// copyright notice included in the Program.  TI reserves all 
// rights not specifically granted under this license. Except 
// as specifically provided herein, nothing in this agreement 
// shall be construed as conferring by implication, estoppel, 
// or otherwise, upon you, any license or other right under any 
// TI patents, copyrights or trade secrets.
//

/****************************************************************************/
/* Includes                                                                 */
/****************************************************************************/

#include "JTAGfunc.h"			// JTAG functions
#include "LowLevelFunc.h"		// low level user functions

/****************************************************************************/
/* Global types                                                             */
/****************************************************************************/

extern word DEVICE;
#pragma warnings=off
extern word CodeArray[];		// holds the target program code
#pragma warnings=on

/****************************************************************************/
/* Main section of Replicator program: User can modify/insert code as needed*/
/****************************************************************************/
/* Main function:
   Upon execution completion, LED blinks for F1xx & F4xx device FET tools 
   when used as the target system(P1.0 and P5.1 drive LEDs).
   The target program is stored in file "Target_Code.s43" (see "CodeArray[]")
   which must be added to the project as an Assembler source code file.
   
   Note: All High Level JTAG Functions are applied here.
*/

void main(void)
{
  word p;				// Data pointer
  word ReadArray[0x40];			// Buffer, used for ReadMemQuick()
  
  InitController();			// Initialize the Host 'F149
  ShowStatus(STATUS_ACTIVE, 0);		// Switch both LEDs on to indicate 
                                        // operation.
  
  InitTarget();				// Initialize Target Board 
  if (!GetDevice())			// Set global DEVICE variable, stop
    ShowStatus(STATUS_ERROR, 1);	// here if invalid JTAG ID or 
                                        // time-out. (error: red LED is ON)

  EraseFLASH(ERASE_SGMT, 0xFE00);	// Segment 0 erase Flash (all types)
  if (!EraseCheck(0xFE00, 0x0100))	// Check segment 0 memory erasure (Fxx2..9)
    ShowStatus(STATUS_ERROR, 2);
  EraseFLASH(ERASE_MASS, 0x1000);	// Mass-Erase Flash (all types)
  if (!EraseCheck(0xF000, 0x0800))	// Check main memory erasure (Fxx2..9)
    ShowStatus(STATUS_ERROR, 3);
  if (!EraseCheck(0x1000, 0x0080))	// Check info memory erasure (Fxx2..9)
    ShowStatus(STATUS_ERROR, 4);
    
  WriteMem(F_BYTE, 0x0200, 0x34);	// Write 2 RAM bytes, read back word
  WriteMem(F_BYTE, 0x0201, 0x12);
  if (ReadMem(F_WORD, 0x0200) != 0x1234)
    ShowStatus(STATUS_ERROR, 5);
  WriteMem(F_WORD, 0x0202, 0x5678);	// Write RAM word, read back 2 bytes
  if (ReadMem(F_BYTE, 0x0202) != 0x78)
    ShowStatus(STATUS_ERROR, 6);
  if (ReadMem(F_BYTE, 0x0203) != 0x56)
    ShowStatus(STATUS_ERROR, 7);  
    
  if (!((DEVICE == 0xF123) || (DEVICE == 0xF413)))	// No Read/WriteQuick
                                                        // possible.
  {      
    WriteMemQuick (0x0200, 0x0010, &CodeArray[0]);	// Write RAM block
	if (!VerifyMem(0x0200, 0x0010, &CodeArray[0]))	// Verify(PSA) RAM of
	  ShowStatus(STATUS_ERROR, 8);                  // target. 
	ReadMemQuick  (0x0200, 0x0010, &ReadArray[0]);	// Read RAM block
	for (p = 0; p < 0x0010; p++)			// Verify(word-for- 
	  if (ReadArray[p] != CodeArray[p])             // word) RAM of target.
	    ShowStatus(STATUS_ERROR, 9);    
  }
   
  if (!WriteFLASHallSections(&CodeArray[0]))		// Write/Verify(PSA) 
    ShowStatus(STATUS_ERROR, 10);                       // all Flash sections.
    
  WriteMem(F_BYTE, P1OUT_, 0x01);	// Control target peripherals directly:
  WriteMem(F_BYTE, P5OUT_, 0x02);	// P1.0 for F1xx devices
  WriteMem(F_BYTE, P1DIR_, 0x01);	// P5.1 for F4xx devices
  WriteMem(F_BYTE, P5DIR_, 0x02);	
  Delay(1000);				// LED on for 1.0s
  WriteMem(F_BYTE, P1OUT_, 0x00);
  WriteMem(F_BYTE, P5OUT_, 0x00);
  Delay(1000);				// LED off for 1.0s
  
  /*****************************************************************************
  Remove following comments to enable fuse blow routine. Blowing of the JTAG
  access protection fuse is irreversible.*/
     
  //if (!BlowFuse())			// Blow security fuse
  //  ShowStatus(STATUS_ERROR, 15);	// ***Be careful with this function!***

  /****************************************************************************/

  ReleaseDevice(V_RESET);	        // Perform Reset, release CPU from JTAG control
  Delay(3000);				// Target application blinks for 3.0s
  ReleaseTarget();			// Release Target Board from power
 
  ShowStatus(STATUS_OK, 0);		// OK: green LED is ON  
  while(1);				// Idle, wait for 'GO' keypress
}

/****************************************************************************/
/*                         END OF SOURCE FILE                               */
/****************************************************************************/
